﻿///////////////////////////////////////////////////////////////////////////////
//
// (C) 2015 ICE TEA GROUP LLC - ALL RIGHTS RESERVED
//
// 
//
// ALL INFORMATION CONTAINED HEREIN IS, AND REMAINS
// THE PROPERTY OF ICE TEA GROUP LLC AND ITS SUPPLIERS, IF ANY.
// THE INTELLECTUAL PROPERTY AND TECHNICAL CONCEPTS CONTAINED
// HEREIN ARE PROPRIETARY TO ICE TEA GROUP LLC AND ITS SUPPLIERS
// AND MAY BE COVERED BY U.S. AND FOREIGN PATENTS, PATENT IN PROCESS, AND
// ARE PROTECTED BY TRADE SECRET OR COPYRIGHT LAW.
//
// DISSEMINATION OF THIS INFORMATION OR REPRODUCTION OF THIS MATERIAL
// IS STRICTLY FORBIDDEN UNLESS PRIOR WRITTEN PERMISSION IS OBTAINED
// FROM ICE TEA GROUP LLC.
//
///////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Design;
using System.Linq;
using System.Web;
using Wisej.Base;
using Wisej.Core;
using WinForms = System.Windows.Forms;

namespace Wisej.Web
{
	/// <summary>
	/// Represents a small rectangular pop-up window that displays a brief description of a control's purpose when the user rests the pointer on the control.
	/// </summary>
	[ToolboxItem(true)]
	[ToolboxBitmap(typeof(WinForms.ToolTip))]
	[ProvideProperty("ToolTip", typeof(Control))]
	[SRDescription("DescriptionToolTip")]
	public class ToolTip : Wisej.Web.Component, IExtenderProvider
	{
		// collection of controls with relative tooltip text.
		private Dictionary<Control, string> tooltips;

		#region Constructors

		/// <summary>
		/// Initializes a new instance of the <see cref="T:Wisej.Web.ToolTip" /> without a specified container.
		/// </summary>
		public ToolTip()
		{
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="T:Wisej.Web.ToolTip" /> class with a specified container.
		/// </summary>
		/// <param name="container">An <see cref="T:System.ComponentModel.IContainer" /> that represents the container of the <see cref="T:Wisej.Web.ToolTip" />. </param>
		public ToolTip(IContainer container)
			: this()
		{
			if (container == null)
				throw new ArgumentNullException("container");

			container.Add(this);
		}

		#endregion

		#region Properties

		/// <summary>
		/// Returns or sets whether the ToolTip provider is currently active.
		/// </summary>
		/// <returns>true if the ToolTip is currently active; otherwise, false. The default is true.</returns>
		[DefaultValue(true)]
		[SRDescription("ToolTipActiveDescr")]
		public bool Active
		{
			get
			{
				return this._active;
			}
			set
			{
				if (this._active != value)
				{
					this._active = value;

					Update();
				}
			}
		}
		private bool _active = true;

		/// <summary>
		/// Returns or sets the automatic delay for the ToolTip.
		/// </summary>
		/// <returns>The automatic delay, in milliseconds. The default is 500.</returns>
		[DefaultValue(500)]
		[RefreshProperties(RefreshProperties.All)]
		[SRDescription("ToolTipAutomaticDelayDescr")]
		public int AutomaticDelay
		{
			get
			{
				return this._automaticDelay;
			}
			set
			{
				if (value < 0)
					throw new ArgumentOutOfRangeException("AutomaticDelay", SR.GetString("InvalidLowBoundArgumentEx", "AutomaticDelay", value));

				if (this._automaticDelay != value)
				{
					this._automaticDelay = value;

					this._initialDelay = value;
					this._autoPopDelay = value * 10;

					Update();
				}

			}
		}
		private int _automaticDelay = 500;

		/// <summary>
		/// Returns or sets the period of time the ToolTip remains visible if the pointer is stationary on a control with specified ToolTip text.
		/// </summary>
		/// <returns>The period of time, in milliseconds, that the <see cref="T:Wisej.Web.ToolTip" /> remains visible when the pointer is stationary on a control. The default value is 5000.</returns>
		[RefreshProperties(RefreshProperties.All)]
		[SRDescription("ToolTipAutoPopDelayDescr")]
		public int AutoPopDelay
		{
			get
			{
				return this._autoPopDelay;
			}
			set
			{
				if (value < 0)
					throw new ArgumentOutOfRangeException("AutoPopDelay", SR.GetString("InvalidLowBoundArgumentEx", "AutoPopDelay", value));

				if (this._autoPopDelay != value)
				{
					this._autoPopDelay = value;

					Update();
				}
			}
		}
		private int _autoPopDelay = 5000;

		private bool ShouldSerializeAutoPopDelay()
		{
			return this._autoPopDelay != (this._automaticDelay * 10);
		}

		private void ResetAutoPopDelay()
		{
			this._autoPopDelay = this._automaticDelay * 10;
		}

		/// <summary>
		/// Returns or sets the time that passes before the ToolTip appears.
		/// </summary>
		/// <returns>The period of time, in milliseconds, that the pointer must remain stationary on a control before the ToolTip window is displayed.</returns>
		[RefreshProperties(RefreshProperties.All)]
		[SRDescription("ToolTipInitialDelayDescr")]
		public int InitialDelay
		{
			get
			{
				return this._initialDelay;
			}
			set
			{
				if (value < 0)
					throw new ArgumentOutOfRangeException("InitialDelay", SR.GetString("InvalidLowBoundArgumentEx", "InitialDelay", value));

				if (this._initialDelay != value)
				{
					this._initialDelay = value;

					Update();
				}
			}
		}
		private int _initialDelay = 500;

		private bool ShouldSerializeInitialDelay()
		{
			return this._initialDelay != this._automaticDelay;
		}

		private void ResetInitialDelay()
		{
			this._initialDelay = this._automaticDelay;
		}

		/// <summary>
		/// Returns or sets the object that contains programmer-supplied data associated with the <see cref="T:Wisej.Web.ToolTip" />.
		/// </summary>
		/// <returns>An <see cref="T:System.Object" /> that contains data about the <see cref="T:Wisej.Web.ToolTip" />. The default is null.</returns>
		[DefaultValue(null)]
		[Localizable(false)]
		[SRCategory("CatData")]
		[SRDescription("ControlTagDescr")]
		[TypeConverter(typeof(StringConverter))]
		public object Tag
		{
			get { return this._tag; }
			set { this._tag = value; }
		}
		private object _tag;

		/// <summary>
		/// Returns or sets a value that defines the type of icon to be displayed alongside the ToolTip text.
		/// </summary>
		/// <returns>One of the <see cref="T:Wisej.Web.ToolTipIcon" /> enumerated values.</returns>
		[DefaultValue(ToolTipIcon.Default)]
		[SRDescription("ToolTipToolTipIconDescr")]
		public ToolTipIcon ToolTipIcon
		{
			get
			{
				return this._toolTipIcon;
			}
			set
			{
				if (this._toolTipIcon != value)
				{
					this._toolTipIcon = value;

					Update();
				}
			}
		}
		private ToolTipIcon _toolTipIcon = ToolTipIcon.Default;

		/// <summary>
		/// Returns or sets a value indicating that the control
		/// can display html in the Text property.
		///</summary>
		[DefaultValue(false)]
		[SRCategory("CatAppearance")]
		[SRDescription("AllowHtmlDescr")]
		public bool AllowHtml
		{
			get
			{
				return this._allowHtml;
			}
			set
			{
				if (this._allowHtml != value)
				{
					this._allowHtml = value;

					Update();
				}
			}
		}
		private bool _allowHtml = false;

		/// <summary>
		/// Returns or sets the alignment side and position of the tooltips.
		/// </summary>
		[DefaultValue(null)]
		[SRCategory("CatLayout")]
		[SRDescription("ToolTipAlignmentDescr")]
		public Placement? Alignment
		{
			get { return this._alignment; }
			set
			{
				if (this._alignment != value)
				{
					this._alignment = value;

					Update();
				}
			}
		}
		private Placement? _alignment = null;

		#endregion

		#region Methods

		/// <summary>
		/// Returns true if the ToolTip can offer an extender property to the specified target component.
		/// </summary>
		/// <returns>true if the <see cref="T:Wisej.Web.ToolTip" /> class can offer one or more extender properties; otherwise, false.</returns>
		/// <param name="target">The target object to add an extender property to. </param>
		public bool CanExtend(object target)
		{
			return (target is Control);
		}

		/// <summary>
		/// Retrieves the ToolTip text associated with the specified control.
		/// </summary>
		/// <returns>A <see cref="T:System.String" /> containing the ToolTip text for the specified control.</returns>
		/// <param name="control">The <see cref="T:Wisej.Web.Control" /> for which to retrieve the <see cref="T:Wisej.Web.ToolTip" /> text. </param>
		[DefaultValue("")]
		[Localizable(true)]
		[DisplayName("ToolTip")]
		[SRDescription("ToolTipToolTipDescr")]
		[Editor("System.ComponentModel.Design.MultilineStringEditor, System.Design, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", typeof(UITypeEditor))]
		public string GetToolTip(Control control)
		{
			if (control == null)
				throw new ArgumentNullException("control");

			string text = string.Empty;
			if (this.tooltips != null)
				this.tooltips.TryGetValue(control, out text);

			return text;
		}

		/// <summary>
		/// Removes all ToolTip text currently associated with the ToolTip component.
		/// </summary>
		public void Clear()
		{
			if (this.tooltips != null)
			{
				this.tooltips.ToList().ForEach((o) => {
					o.Key.Disposed -= this.Control_Disposed;
					o.Key.ControlCreated -= this.Control_Created;
				});

				this.tooltips.Clear();

				Update();
			}
		}

		/// <summary>
		/// Associates ToolTip text with the specified control.
		/// </summary>
		/// <param name="control">The <see cref="T:Wisej.Web.Control" /> to associate the ToolTip text with. </param>
		/// <param name="caption">The ToolTip text to display when the pointer is on the control. </param>
		public void SetToolTip(Control control, string caption)
		{
			if (control == null)
				throw new ArgumentNullException("control");

			if (String.IsNullOrEmpty(caption))
			{
				if (this.tooltips != null)
					this.tooltips.Remove(control);
			}
			else
			{
				if (this.tooltips == null)
					this.tooltips = new Dictionary<Control, string>();

				this.tooltips[control] = caption;

				// remove the control from the extender when it's disposed.
				control.Disposed -= this.Control_Disposed;
				control.Disposed += this.Control_Disposed;
			}

			Update();
		}

		private void Control_Disposed(object sender, EventArgs e)
		{
			Control control = (Control)sender;
			control.Disposed -= this.Control_Disposed;
			control.ControlCreated -= this.Control_Created;

			// remove the extender values associated with the disposed control.
			if (this.tooltips != null)
				this.tooltips.Remove(control);
		}

		private void Control_Created(object sender, EventArgs e)
		{
			// handle the delayed registration of this extender for a control
			// that was not created (not visible) when the extender tried to register it.
			Control control = (Control)sender;
			control.ControlCreated -= this.Control_Created;

			// update the extender, now it will send also this newly created control.
			Update();
		}

		protected override void Dispose(bool disposing)
		{
			if (disposing)
			{
				Clear();
			}

			base.Dispose(disposing);
		}

		/// <summary>
		/// Returns a string representation for this control.
		/// </summary>
		/// <returns>A <see cref="T:System.String" /> containing a description of the <see cref="T:Wisej.Web.ToolTip" />.</returns>
		public override string ToString()
		{
			return string.Concat(
				base.ToString(),
				" InitialDelay: ", this.InitialDelay);
		}

		#endregion

		#region Wisej Implementation

		/// <summary>
		/// Renders the client component.
		/// </summary>
		/// <param name="config">Dynamic configuration object.</param>
		protected override void OnWebRender(dynamic config)
		{
			base.OnWebRender((object)config);

			config.className = "wisej.web.extender.ToolTip";
			config.active = this.Active;
			config.icon = this.ToolTipIcon;
			config.popDelay = this.AutoPopDelay;
			config.alignment = this.Alignment;
			config.initialDelay = this.InitialDelay;

			if (this.tooltips != null)
			{
				// send only created controls.
				config.tooltips = this.tooltips
					.Where(o => o.Key.Created)
						.Select(o => new
						{
							id = ((IWisejControl)o.Key).Id,
							text = TextUtils.EscapeText(o.Value, this.AllowHtml)

						}).ToArray();

				// register non-created control for delayed registration.
				this.tooltips.Where(o => !o.Key.Created).ToList().ForEach(o =>
				{
					Control control = o.Key;
					control.ControlCreated -= this.Control_Created;
					control.ControlCreated += this.Control_Created;
				});
			}
		}

		#endregion

	}
}
